package sscce;
import java.io.File;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;
import java.util.Random;
import java.util.concurrent.CountDownLatch;

import javax.jms.BytesMessage;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.naming.Context;
import javax.naming.InitialContext;

/**
 * see http://community.jboss.org/message/612334#612334
 */

public class HornetQPagingTest
{
 // === Static ================================================================
 
 private static final String HQURL = "jnp://ttwsrvmi09:61099";
 private static final String QUEUE_IN = "queue/inbound_messages";
 private static final long   LOG_INTERVAL = 5000L;
 private static final long   STARTTIME = System.currentTimeMillis();
 
 public static void main(String[] args)
 throws Exception
 {
  int            senderCount = 1, senderMessageCount = 1000000,
                 receiverCount = 1, receiverMsgCount = 1000000,
                 receiverDelay = 45000;
  List<Thread>   senders, receivers;
  CountDownLatch startLatch, stopLatch;
  
  startLatch = new CountDownLatch(senderCount + receiverCount);
  stopLatch  = new CountDownLatch(senderCount + receiverCount);
  
  senders   = new LinkedList<Thread>();
  receivers = new LinkedList<Thread>();
  
  for(int id = 0; id < senderCount; id ++)
  {
   senders.add(new Thread(new Sender(id, startLatch, stopLatch, HQURL, QUEUE_IN, 1024, senderMessageCount)));
  }
  
  for(int id = 0; id < receiverCount; id ++)
  {
   receivers.add(new Thread(new Receiver(id, startLatch, stopLatch, HQURL, QUEUE_IN, receiverMsgCount, receiverDelay)));
  }  
  
  log("Main: Starting threads ...");
  
  for(Thread t : senders)   t.start();
  for(Thread t : receivers) t.start();
  
  log("Main: Joining threads ...");
  
  for(Thread t : senders)   t.join();
  for(Thread t : receivers) t.join();
 }

 private static final void log(final String text)
 {
 	System.out.printf("[%8d] %s\n", System.currentTimeMillis() - STARTTIME, text);
 }
 
 private static class Sender implements Runnable
 {
  private final String hqURL, queueName;
  private final CountDownLatch startLatch, stopLatch;
  private final int msgSize, msgCount, id;
    
  public Sender(final int id, final CountDownLatch startLatch, final CountDownLatch stopLatch, final String hqURL, final String queueName, final int msgSize, final int msgCount)
  {
   this.id         = id;
   this.startLatch = startLatch;
   this.stopLatch  = stopLatch;
   this.hqURL      = hqURL;
   this.queueName  = queueName;
   this.msgSize    = msgSize;
   this.msgCount   = msgCount;
  }
  
  @Override
		public void run()
  {
   long              time, lastRepTime, lastRepCount;
   
   Properties        jndiProperties;
   Context           initialContext = null;
   Queue             queue;
   ConnectionFactory connectionFactory;
   Connection        connection = null;
   Session           session;
   MessageProducer   producer;
   BytesMessage      message;
   
   byte[]            msgContent;
   
   try
   {
    jndiProperties = new Properties();
    jndiProperties.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
    jndiProperties.setProperty(Context.PROVIDER_URL,            hqURL);
  
    initialContext = new InitialContext(jndiProperties);
    
    queue             = (Queue)initialContext.lookup(queueName);
    connectionFactory = (ConnectionFactory)initialContext.lookup("ConnectionFactory");
  
    connection = connectionFactory.createConnection();
    session    = connection.createSession(true, 0);
    producer   = session.createProducer(queue);
    
    connection.start();
    
    msgContent = new byte[msgSize]; (new Random()).nextBytes(msgContent);
    
    log("S#" + id + " ready ...");
    startLatch.countDown(); startLatch.await();
    log("S#" + id + " running ...");
    
    time         = System.currentTimeMillis();
    lastRepTime  = System.currentTimeMillis();
    lastRepCount = 0;
    
    for(int msgId = 0; msgId < msgCount; msgId ++)
    {
     message = session.createBytesMessage();
     message.writeBytes(msgContent);
     
     producer.send(message);
     session.commit();
     
     if(lastRepTime < System.currentTimeMillis() - LOG_INTERVAL)
     {
      log("S#" + id + ": MsgCount: " + msgId + " Speed: " + (msgId - lastRepCount) * 1000 / (System.currentTimeMillis() - lastRepTime) + " msg/s");
      lastRepTime = System.currentTimeMillis();
      lastRepCount = msgId;
      
      while((new File("stop.send")).exists())
      {
       log("S#" + id + ": Sleeping");
       Thread.sleep(1000);
      }
     }
    }
    
    time = System.currentTimeMillis() - time;
    
    log("S#" + id + ": Waiting for latch ...");
    stopLatch.countDown(); stopLatch.await();
    log("S#" + id + ": MsgCount: " + msgCount + " Time: " + time + "ms; " + (msgCount * 1000L / time) + "msg/s");
   }
   catch(Exception e)
   {
    e.printStackTrace();
   }
   finally
   {
    if(initialContext != null) try { initialContext.close(); } catch(Exception e) { /* NOP */ }
    if(connection     != null) try { connection.close();     } catch(Exception e) { /* NOP */ }
   }
  }  
 }
 
 private static class Receiver implements Runnable
 {
  private final String hqURL, queueName;
  private final CountDownLatch startLatch, stopLatch;
  private final int id, msgCount, receiverDelay;
    
  public Receiver(final int id, final CountDownLatch startLatch, final CountDownLatch stopLatch, final String hqURL, final String queueName, final int msgCount, final int receiverDelay)
  {
   this.id            = id;
   this.startLatch    = startLatch;
   this.stopLatch     = stopLatch;
   this.hqURL         = hqURL;
   this.queueName     = queueName;
   this.msgCount      = msgCount;
   this.receiverDelay = receiverDelay;
  }
  
  @Override
		public void run()
  {
   long              time, lastRepTime, lastRepCount;
   
   Properties        jndiProperties;
   Context           initialContext = null;
   Queue             queue;
   ConnectionFactory connectionFactory;
   Connection        connection = null;
   Session           session;
   MessageConsumer   consumer;
   
   try
   {
    jndiProperties = new Properties();
    jndiProperties.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
    jndiProperties.setProperty(Context.PROVIDER_URL,            hqURL);
  
    initialContext = new InitialContext(jndiProperties);
    
    queue             = (Queue)initialContext.lookup(queueName);
    connectionFactory = (ConnectionFactory)initialContext.lookup("ConnectionFactory");
  
    connection = connectionFactory.createConnection();
    session    = connection.createSession(true, 0);
    consumer   = session.createConsumer(queue);
    
    connection.start();
    
    log("R#" + id + " ready ...");
    startLatch.countDown(); startLatch.await();
    if(receiverDelay > 0) Thread.sleep(receiverDelay);
    log("R#" + id + " running ...");
    
    time         = System.currentTimeMillis();
    lastRepTime  = System.currentTimeMillis();
    lastRepCount = 0;  
    
    for(int msgId = 0; msgId < msgCount; msgId ++)
    {
     consumer.receive();
     session.commit();
     
     if(lastRepTime < System.currentTimeMillis() - LOG_INTERVAL)
     {
      log("R#" + id + ": MsgCount: " + msgId + " Speed: " + (msgId - lastRepCount) * 1000 / (System.currentTimeMillis() - lastRepTime) + " msg/s");
      lastRepTime = System.currentTimeMillis();
      lastRepCount = msgId;
     }     
    }
    
    time = System.currentTimeMillis() - time;
    
    log("R#" + id + ": Waiting for latch ...");
    stopLatch.countDown(); stopLatch.await();
    log("R#" + id + ": MsgCount: " + msgCount + " Time: " + time + "ms; " + (msgCount * 1000L / time) + "msg/s");
   }
   catch(Exception e)
   {
    e.printStackTrace();
   }
   finally
   {
    if(initialContext != null) try { initialContext.close(); } catch(Exception e) { /* NOP */ }
    if(connection     != null) try { connection.close();     } catch(Exception e) { /* NOP */ }
   }
  }  
 } 
}
